﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Windows.Data;
using System.Windows.Input;
using GalaSoft.MvvmLight.Command;
using SpotnetWatchdog.Entities;
using SpotnetWatchdog.Utils;

namespace SpotnetWatchdog.ViewModels
{
    /// <summary>
    /// View model for the list with watch items
    /// </summary>
    class WatchItemsViewModel : ViewModelBase
    {
        #region Fields

        private List<Category> _categories;
        private WatchItem _currentWatchItem;
        private IWatchdogRepository _watchdogRepository;
        private ObservableCollection<WatchItem> _watchItems;

        #endregion

        #region Constructors

        public WatchItemsViewModel()
        {
            _watchdogRepository = WatchdogRepository.Instance;
            CurrentWatchItem = new WatchItem();

            InitializeCommands();
            BeginLoadData();
        }

        #endregion

        #region Public Properties

        /// <summary>
        /// Gets or sets Categories
        /// </summary>
        public List<Category> Categories
        {
            get { return _categories; }
            set
            {
                if (_categories != value)
                {
                    _categories = value;
                    RaisePropertyChanged("Categories");
                }
            }
        }

        /// <summary>
        /// Gets or sets CurrentWatchItem
        /// </summary>
        public WatchItem CurrentWatchItem
        {
            get { return _currentWatchItem; }
            set
            {
                if (_currentWatchItem != value && (_currentWatchItem == null || value == null || _currentWatchItem.Id != value.Id))
                {
                    _currentWatchItem = WatchItem.Copy(value);
                    RaisePropertyChanged("CurrentWatchItem");
                }
            }
        }

        /// <summary>
        /// Gets or sets WatchItems
        /// </summary>
        public ObservableCollection<WatchItem> WatchItems
        {
            get { return _watchItems; }
            private set
            {
                if (_watchItems != value)
                {
                    _watchItems = value;
                    RaisePropertyChanged("WatchItems");
                }
            }
        }

        #endregion

        #region Commands

        private void InitializeCommands()
        {
            DeleteWatchItemCommand = new RelayCommand(BeginDeleteWatchItemCommand, () => !IsBusy && CurrentWatchItem != null);
            NewWatchItemCommand = new RelayCommand(() => CurrentWatchItem = new WatchItem(), () => !IsBusy);
            SaveWatchItemCommand = new RelayCommand(BeginSaveWatchItem, () => !IsBusy && CurrentWatchItem != null && CurrentWatchItem.IsValid());
        }

        #region Delete Watch Item

        public ICommand DeleteWatchItemCommand { get; private set; }

        private void BeginDeleteWatchItemCommand()
        {
            IsBusy = true;

            WatchItem itemInList = WatchItems.SingleOrDefault((item) => item.Id == CurrentWatchItem.Id);

            //if the current watch item exists in the list
            if (itemInList != null)
            {
                WatchItems.Remove(itemInList);

                BackgroundWorker backgroundWorker = new BackgroundWorker();
                backgroundWorker.DoWork += (sender, e) => _watchdogRepository.SaveWatchItems();
                backgroundWorker.RunWorkerCompleted += DeleteWatchItems_Completed;

                backgroundWorker.RunWorkerAsync(WatchItems);
            }
            else
            {
                CurrentWatchItem = new WatchItem();

                IsBusy = false;
            }
        }

        private void DeleteWatchItems_Completed(object sender, RunWorkerCompletedEventArgs e)
        {
            if (WatchItems.Count == 0) CurrentWatchItem = new WatchItem();

            IsBusy = false;

            if (e.Error != null)
            {
                MessageDialog.ShowErrorMessage(e.Error.Message);
            }
        }

        #endregion

        #region New Watch Item

        public ICommand NewWatchItemCommand { get; private set; }

        #endregion

        #region Save Watch Item

        public ICommand SaveWatchItemCommand { get; private set; }

        private void BeginSaveWatchItem()
        {
            IsBusy = true;

            WatchItem itemInList = WatchItems.SingleOrDefault((item) => item.Id == CurrentWatchItem.Id);
            //if the current watch item is new
            if (itemInList == null)
            {
                WatchItems.Add(CurrentWatchItem);
            }
            else
            {
                WatchItem.Map(CurrentWatchItem, itemInList);
            }

            BackgroundWorker backgroundWorker = new BackgroundWorker();
            backgroundWorker.DoWork += (sender, e) => _watchdogRepository.SaveWatchItems();
            backgroundWorker.RunWorkerCompleted += SaveWatchItems_Completed;

            backgroundWorker.RunWorkerAsync(WatchItems);
        }

        private void SaveWatchItems_Completed(object sender, RunWorkerCompletedEventArgs e)
        {
            IsBusy = false;

            if (e.Error != null)
            {
                MessageDialog.ShowErrorMessage(e.Error.Message);
            }
        }

        #endregion

        #endregion

        #region Load Data

        private void BeginLoadData()
        {
            IsBusy = true;

            BackgroundWorker backgroundWorker = new BackgroundWorker();
            backgroundWorker.DoWork += LoadData;
            backgroundWorker.RunWorkerCompleted += LoadData_Completed;
            backgroundWorker.RunWorkerAsync();
        }

        private void LoadData(object sender, DoWorkEventArgs e)
        {
            e.Result = new Data()
            {
                Categories = _watchdogRepository.Categories,
                WatchItems = _watchdogRepository.WatchItems,
            };
        }

        private void LoadData_Completed(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                IsBusy = false;

                MessageDialog.ShowErrorMessage(e.Error.Message);
            }
            else
            {
                Data data = (Data)e.Result;
                Categories = data.Categories;
                WatchItems = data.WatchItems;
                
                ListCollectionView watchItemsView = (ListCollectionView)CollectionViewSource.GetDefaultView(WatchItems);
                watchItemsView.SortDescriptions.Add(new SortDescription("Description", ListSortDirection.Ascending));

                CurrentWatchItem = WatchItems.FirstOrDefault() ?? new WatchItem();

                IsBusy = false;
            }
        }

        private class Data
        {
            public List<Category> Categories { get; set; }
            public ObservableCollection<WatchItem> WatchItems { get; set; }
        }

        #endregion
    }
}
